#include <Pilot.h>
#include "globals.h"
#include "pagedisplay.h"
#include "2bit.h"
#include "notify.h"



/* The size of the main drawable part of the screen */
static const SWord winXextent = MAIN_DISPLAY_WIDTH,
    winYextent = MAIN_DISPLAY_HEIGHT;

static SWord INTWinProxyString(VoidHand hText, int xScrollOffset,
    int yScrollOffset, RectanglePtr r1, RectanglePtr r2, int mode);
static SWord INTWinProxyStringPiece(SWord *startPoint, int xScrollOffset,
    int yScrollOffset, RectanglePtr r1, RectanglePtr r2, int mode);
static int INTPieceClicked(SWord *startPoint, SWord xClick, SWord yClick, 
    int xScrollOffset, int yScrollOffset, SWord *xcoordp, SWord *ycoordp);

/****************************************************************************
 *
 * * Proxy String Format
 *
 * A proxy string is a string resource which has text information encoded
 * in the proxy string format which is defined as follows:
 *
 *   string - an entire string resource, contains zero or more pieces,
 *            followed by an end token of 0xFFFF (16 bits)
 *   piece  - a subpart of the string resource, contains zero or more frags,
 *            followed by an end token of 0x0000 (16 bits)
 *            each piece is associated with attribute and position data
 *   frag   - a subpart of a piece, composed of a 16-bit length followed by
 *            8-bit chars
 *            (the number of which should be equal to the length, except when
 *             the length is odd in which one blank char is added to maintain
 *             16-bit word-alignment for purposes of addressing)
 *
 * (admittedly, this nomenclature sucks)
 *
 * graphically speaking:
 *
 * string:
 * +--------+--------+
 * | pieces | 0xFFFF |
 * | x byte | 2 byte |
 * +--------+--------+
 *
 * piece:
 * +--------+--------+--------+--------+--------+--------+--------+--------+
 * | Back*  | For*   | Xpos   | Ypos   | Xextnt | Yextnt | Attrib | data   |
 * | 2 byte | 2 byte | 2 byte | 2 byte | 2 byte | 2 byte | 2 byte | x byte |
 * +--------+--------+--------+--------+--------+--------+--------+--------+
 * (lines have no data, Xextnt and Yextnt are the x and y lengths of the line)
 *
 * data:
 * (when it is a line of text)
 * +--------+--------+         
 * | frag(s | 0x0000 | 
 * | x byte | 2 byte |  
 * +--------+--------+
 *
 * (or when the 5th bit is on, meaning a bitmap)
 * +--------+
 * | bitmap |
 * | x byte |
 * +--------+
 *
 * (or when it is a link) 
 * +--------+--------+--------+--------+
 * | length | link   | frag(s | 0x0000 |
 * | 2 byte | x byte | x byte | 2 byte |
 * +--------+--------+--------+--------+
 *
 * (link must be a NULL terminated string, the NULL is included in the length)
 * (length/link pair may be repeated for use as an LINK_ALT)
 *
 * frag:
 * (when it is not a link
 * +--------+--------+
 * | length | char(s |
 * | 2 byte | x byte |
 * +--------+--------+
 * 
 * (as a general rule, Y position are constructed increasing order)
 *
 * ==> where x is always a multiple of 2 bytes to maintain 16-bit alignment <==
 *
 * attribute masks: (see #defines below for actual values)
 *   BOLD       0x1
 *   ITALIC     0x2
 *   UNDERLINE  0x4
 *   SIZE       0x18
 *   IMAGE      0x20  (IMAGE|BOLD means 2-bit greyscale)
 *   LINK       0x40
 *   LINE       0x80  
 *
 *
 * minimal example:
 * byte#  field
 *  0     BackPtr (SWord), an offset to the previous BackPtr
 *  2     ForPtr  (SWord), an offset to the next ForPtr
 *  4     Xpos    (SWord), starting X coord on drawing surface (see note)
 *  6     Ypos    (SWord), starting Y coord on drawing surface (see note)
 *  8     Attrib  (SWord), text attributes, see below
 * 10     Length  (SWord), length of line string
 * 12-XX  Text, must be word (16-bit) aligned
 *          text of odd length is padded with an blank char
 * XX+1   Piece End Token (0x0000)
 * XX+3   String End Token (0xFFFF)
 *
 * Note: Xpos and Ypos are relative to an page absolute coordinate, which is
 *       independent of the screen on the pilot
 *
 ***************************************************************************/ 

/***************************************************************************
 *
 * Proxy HTML "Page" Resource Layout (currently unused) 
 * 
 * Resources are in a database of type 'PRpg' (Proxy Page)
 *                             creator 'TGwm' (Top Gun Wing Man)
 *
 * 
 * ID Number Allocation
 *
 * String Segment is ID 1999 only
 * Bitmap segment 2000 thru 2999 (currently unimplemented)
 * (more segments to be defined as needed)
 *
 * The browser will do layout by scanning thru each of the segments
 * consecutively. On first lookup miss, it will assume it has finished
 * reading all the segments
 *
 ***************************************************************************/

/****************************************************************************
 *
 * Text Drawing / Erasing Functions
 *
 ****************************************************************************/

SWord WinDrawProxyString(VoidHand hText, int xScrollOffset, int yScrollOffset)
{
  RectangleType r;
  r.topLeft.x = 0;
  r.topLeft.y = 0;
  r.extent.x = winXextent;
  r.extent.y = winYextent;
  return INTWinProxyString(hText, xScrollOffset, yScrollOffset, &r, NULL, 0);
}

SWord WinDrawProxyStringGnomon(VoidHand hText, int xScrollOffset,
    int yScrollOffset, RectanglePtr r1, RectanglePtr r2)
{
  return INTWinProxyString(hText, xScrollOffset, yScrollOffset, r1, r2, 0);
}

SWord WinEraseProxyString(VoidHand hText, int xScrollOffset, int yScrollOffset)
{
  /* this function is probably unnecessary since WinEraseWindow does a
   * more complete job, but it here in case you wanted to do anything cool
   */
  RectangleType r;
  r.topLeft.x = 0;
  r.topLeft.y = 0;
  r.extent.x = winXextent;
  r.extent.y = winYextent;
  return INTWinProxyString(hText, xScrollOffset, yScrollOffset, &r, NULL, 1);
}

/* Erase the main part of the screen */
void EraseMainWindow(void)
{
    RectangleType r;
    r.topLeft.x = 0;
    r.topLeft.y = 0;
    r.extent.x = winXextent;
    r.extent.y = winYextent;
    WinEraseRectangle(&r, 0);
}

/****************************************************************************
 *
 * Internal functions for Win???ProxyString
 *
 * as a rule, you shouldn't call them
 * 
 ****************************************************************************/

static SWord INTWinProxyString(VoidHand hText, int xScrollOffset,
    int yScrollOffset, RectanglePtr r1, RectanglePtr r2, int mode)
{
  SWord *proxyString = MemHandleLock(hText);
  /* int byteSize = MemPtrSize(proxyString); fault tolerance test */
  int nextParseOffset = 0, retVal;
  RectangleType oldclip, newclip;

  /* Set a clipping region */
  WinGetClip(&oldclip);
  newclip.topLeft.x = 0;
  newclip.topLeft.y = 0;
  newclip.extent.x = winXextent;
  newclip.extent.y = winYextent;
  WinSetClip(&newclip);

  if (r1 == NULL && r2 == NULL) r1 = &newclip;

  while (proxyString[nextParseOffset] != 0xFFFF) {
    retVal = INTWinProxyStringPiece(&proxyString[nextParseOffset],
				    xScrollOffset, yScrollOffset,
				    r1, r2, mode);
    if (retVal == -1) { 
      break; 
    } else {
      nextParseOffset += retVal;
    }
  }
  WinSetUnderlineMode(noUnderline);
  MemHandleUnlock(hText);
  WinSetClip(&oldclip);
  return 0;
}

static SWord INTWinProxyStringPiece(SWord *startPoint, int xScrollOffset,
    int yScrollOffset, RectanglePtr r1, RectanglePtr r2, int mode)
{
  /* function also deals with frags internally */
  int currParseLoc = 0;
  int currFragLen = 0; /* used to determine num of chars left in current frag */
  int internalOffset = 0;
  int retVal;
  SWord forwardOffset = 0, backOffset = 0;
  SWord startXPos = 0, startYPos = 0;
  SWord Xextent = 0, Yextent = 0;
  Word attributes = 0;
  /* internal preprocessing of window attributes */

  /* read metadata and then move it to the next read pos */
  backOffset = startPoint[0];
  forwardOffset = startPoint[1];
  startXPos = startPoint[2];
  startYPos = startPoint[3];
  Xextent = startPoint[4];
  Yextent = startPoint[5];
  attributes = (Word) startPoint[6];
  currParseLoc = 7;

  if ((r1 &&
    startXPos-xScrollOffset <= r1->topLeft.x + r1->extent.x &&
    startXPos-xScrollOffset+Xextent > r1->topLeft.x &&
    startYPos-yScrollOffset <= r1->topLeft.y + r1->extent.y &&
    startYPos-yScrollOffset+Yextent > r1->topLeft.y) ||
      (r2 &&
    startXPos-xScrollOffset <= r2->topLeft.x + r2->extent.x &&
    startXPos-xScrollOffset+Xextent > r2->topLeft.x &&
    startYPos-yScrollOffset <= r2->topLeft.y + r2->extent.y &&
    startYPos-yScrollOffset+Yextent > r2->topLeft.y)) {
      /* reduce the amount of redrawing */
  
    if (attributes == LINE) {
      WinDrawLine(startXPos - xScrollOffset,
                  startYPos - yScrollOffset + internalOffset, 
		  startXPos - xScrollOffset + Xextent - 1,
		  startYPos - yScrollOffset + internalOffset + Yextent - 1);
    }
    if ((attributes & IMAGE) && (attributes & IMAGE_HAS_SERV_SIZE)) {
	/* Skip over the "real" x and y size */
	currParseLoc += 2;
    }
    if (attributes & LINK) {
      currFragLen = startPoint[currParseLoc++];
      /* use like this to get a link:
	 char aLink = (char*) &startPoint[currParseLoc]; */    
      /* increment past the link */
      if ((currFragLen % 2) == 0) {
	currParseLoc += (currFragLen >> 1) /* to the next read pos */;
      } else {
	currParseLoc += ((currFragLen + 1) >> 1) /* to the next read pos */;
      }
      WinSetUnderlineMode(grayUnderline);
    }
    if (attributes & LINK_ALT) {
      currFragLen = startPoint[currParseLoc++];
      /* use like this to get a link:
	 char aLink = (char*) &startPoint[currParseLoc]; */    
      /* increment past the link */
      if ((currFragLen % 2) == 0) {
	currParseLoc += (currFragLen >> 1) /* to the next read pos */;
      } else {
	currParseLoc += ((currFragLen + 1) >> 1) /* to the next read pos */;
      }
    }
  
    if (attributes & IMAGE) {
      BitmapPtr image;
      image = (BitmapPtr) &startPoint[currParseLoc];
      if (attributes & IMAGE_2BIT) {
#ifndef NOTWOBIT
	WinDraw2Bitmap(image, startXPos - xScrollOffset,
				startYPos - yScrollOffset);
#else
	WinDrawBitmap(image, startXPos - xScrollOffset,
				startYPos - yScrollOffset);
#endif
      } else {
	WinDrawBitmap(image, startXPos - xScrollOffset,
				startYPos - yScrollOffset);
      }
    } else {
      /* processing of display attributes except underline*/
      if (attributes & BOLD) {
	FntSetFont(boldFont);
      }
      if (attributes & ITALIC) {
	/* Currently unsupported */
      }
      if (attributes & FONT_MASK) {
	if ((attributes & FONT_MASK) == FONT_STD) {
	  FntSetFont(stdFont);
	} else if ((attributes & FONT_MASK) == FONT_BIG1) {
	  FntSetFont(largeFont);
	} else if ((attributes & FONT_MASK) == FONT_BIG2) {
	  /* Currently unsupported */
	} else if ((attributes & FONT_MASK) == FONT_BIG3) {
	  /* currently unsupported */
	}
      }
      /* this attribute is independent of the others on the pilot */
      if ((attributes & UNDERLINE) && !(attributes & LINK)) {
	WinSetUnderlineMode(solidUnderline);
      }
    
      while(1) {
	/* this part would by INTWinProxyStringFrag, but it's rolled in */
	currFragLen = startPoint[currParseLoc++];
      
      /* is there any more to draw and is there still room on screen? */
	if (currFragLen == 0 ||
	    (startYPos - yScrollOffset) > winYextent) {
	  break;
	}
	else {
	  /* yes, so its a proxy string frag */      
	  /* if attempts to draw optimized +100
	     is so that alignment works off 100 */
	  if (startYPos - yScrollOffset + internalOffset + 100> 0) {
	    if (mode) {
	      /* erase mode */
	      WinEraseChars((char*) &startPoint[currParseLoc],
			    (Word) currFragLen, 
			    startXPos - xScrollOffset,
			    startYPos - yScrollOffset + internalOffset);
	    } else {
	      /* draw mode */
	      WinDrawChars((char*) &startPoint[currParseLoc],
			   (Word) currFragLen, 
			   startXPos - xScrollOffset,
			   startYPos - yScrollOffset + internalOffset);
	    }
	  }
	  /* how much do we increment the pointer */
	  if ((currFragLen % 2) == 0) {
	    currParseLoc += (currFragLen >> 1) /* to the next read pos */;
	  } else {
	    currParseLoc += ((currFragLen + 1) >> 1) /* to the next read pos */;
	  }
	
	  /* start next line */
	  internalOffset += FntLineHeight();
	}
      }
      /* clean up */
      FntSetFont(stdFont);
      WinSetUnderlineMode(noUnderline);
    }
  }
  lastLayoutYExtent = startYPos - yScrollOffset + Yextent;
  /* are we at the bottom of the screen? */
  WinSetUnderlineMode(noUnderline);
  if (startYPos - yScrollOffset > winYextent ) {
    retVal = -1; /* yes, so stop drawing */    
  } else {
    retVal = (forwardOffset / 2); /* no, so give me the next piece */
  }
  return retVal;
}

/* PieceClicked returns a pointer to the piece that recieved a click event 
 * search in linear from top, thus region conflicts are resolved by what's first
 * returns -1 if there is no region */

int PieceClicked(VoidHand hText, SWord xClick, SWord yClick, int xScrollOffset,
    int yScrollOffset, SWord *xcoordp, SWord *ycoordp)
{
  SWord *proxyString;
  int nextParseOffset = 0, iterResult;
  int retVal;

  if (xClick > winXextent || yClick > winYextent) return -1;

  proxyString = MemHandleLock(hText);
  if (proxyString[nextParseOffset] == 0xFFFF) {
    retVal = -1;
  } else {
   while (1) {
    /* scan the piece */
    iterResult = INTPieceClicked(&proxyString[nextParseOffset],
				 xClick, yClick, xScrollOffset,
				 yScrollOffset, xcoordp, ycoordp);
    if (iterResult == -1) {
      /* if we've found the piece */
      retVal = nextParseOffset;
      break;
    }
    else {
      /* look at the next piece */
      nextParseOffset += iterResult;
    }

    if (iterResult == -2 ||
	proxyString[nextParseOffset] == 0xFFFF) {
      /* but first make sure there is another piece */
      retVal = -1;
      break; 
    }
   }
  }
  MemHandleUnlock(hText);
  return retVal;
}

static int INTPieceClicked(SWord *startPoint, SWord xClick, SWord yClick,
    int xScrollOffset, int yScrollOffset, SWord *xcoordp, SWord *ycoordp)
{
  /* note to self, since forward pointers between 0 and -6 should be meaningless,
     they are using in the return value as result codes; these ARE NOT offsets */

  /* read metadata and then move it to the next read pos */
  /* SWord backOffset = startPoint[0]; */
  SWord forwardOffset = startPoint[1];
  SWord startXPos = startPoint[2] - xScrollOffset;
  SWord startYPos = startPoint[3] - yScrollOffset;
  SWord Xextent = startPoint[4];
  SWord Yextent = startPoint[5];
  int retVal;

  if (xClick >= startXPos && xClick < (startXPos + Xextent) &&
      yClick >= startYPos && yClick < (startYPos + Yextent)) {
    /* it's in this region */
    if (xcoordp) *xcoordp = xClick - startXPos;
    if (ycoordp) *ycoordp = yClick - startYPos;
    retVal = -1;
  } else if ((startYPos > winYextent) || (startYPos > yClick)) {
    /* about if we're off screen or already past the click */
    retVal = -2;
  } else {
    /* it's not, look in the next one */
    retVal = (forwardOffset / 2);
  }
  return retVal;
}

/* Returns -1 if no popup happened, 0 if Cancel, 1 if the first URL, 2 if
   the second */
int ClickImage(EventPtr pen, SWord *eventPiece, int xScrollOffset,
    int yScrollOffset)
{
  int retval = -1;
  Word attributes;
  SWord startXPos = 0, startYPos = 0;
  SWord Xextent = 0, Yextent = 0;

  startXPos = eventPiece[2];
  startYPos = eventPiece[3];
  Xextent = eventPiece[4];
  Yextent = eventPiece[5];
  attributes = eventPiece[6];
  
  /* if there is an image with some type of link */
  if ((attributes & IMAGE) &&
      ((attributes & LINK) || (attributes & LINK_ALT))) {

    int selection, PopLinkCnt = 0;
    char *PopUpLink1 = "Goto link";
    char *PopUpLink2 = "View full size ";
    char *PopUpLink3 = "Cancel";
    char *PopUpLink[3];
    char *comparison;
    ListPtr pList = FrmGetObjectPtr(FrmGetFormPtr(MainForm),
				    FrmGetObjectIndex(FrmGetFormPtr(MainForm),
						      ImageList));
    if (attributes & LINK) {
      StatusBarText((unsigned char *)
	&eventPiece[(attributes & IMAGE_HAS_SERV_SIZE) ? 10 : 8], 1);
    }

    if (attributes & LINK) {
      PopUpLink[PopLinkCnt++] = PopUpLink1;
    }
    if (attributes & LINK_ALT) {
      PopUpLink[PopLinkCnt++] = PopUpLink2;
    }
    PopUpLink[PopLinkCnt++] = PopUpLink3;

    LstSetHeight(pList, PopLinkCnt);
    LstSetListChoices(pList, PopUpLink, PopLinkCnt);
    LstSetSelection(pList, 0);
    LstSetPosition(pList, (pen->screenX - 5), (pen->screenY - 5));
    retval = 0;
    selection = LstPopupList(pList);
    if (selection != -1) {
      comparison = LstGetSelectionText(pList, selection);
      if (comparison) {
	if (StrCompare(PopUpLink1, comparison) == 0) {
	  retval = 1; 
	} else if (StrCompare(PopUpLink2, comparison) == 0) {
	  if (attributes & LINK) {
	    retval = 2; 
	  } else {
	    retval = 1;
	  }
	} else {
	  retval = 0;
	}
      }
    }
  }
  INTWinProxyString(hPage, xScrollOffset, yScrollOffset, NULL, NULL, 0);
  return retval;
}

/* event = -1 on penDown, 0 on penMove, 1 on penUp, -2 when hPage changes.
   On penUp, returns 1 if the piece was clicked (penDown and penUp on same
   piece without moving off), 0 otherwise.
   On penDown, returns 1 if the piece needed nothing done on penDown, 0
   otherwise. */
int UpdateHiLiPiece(int event, int eventPieceOff, int xScrollOffset,
    int yScrollOffset)
{
  Word attributes = 0;
  SWord forwardOffset = 0, backOffset = 0;
  SWord startXPos = 0, startYPos = 0;
  SWord Xextent = 0, Yextent = 0;
  RectangleType drawRegion;
  int retval = 0;
  SWord *eventPiece;
  SWord *hPtr;

  if (event == -2) {
    /* This is called when hPage changes; it means to forget about any
       piece we may have remembered */
    penDownOrigPieceOff = -1;
    penDownOrigImageFlag = 0;
    return 0;
  }

  hPtr = MemHandleLock(hPage);
  ErrFatalDisplayIf(!hPtr, "Cannot lock hPage");
  eventPiece = eventPieceOff >= 0 ? hPtr + eventPieceOff : NULL;

  if (eventPiece) {
    backOffset = eventPiece[0];
    forwardOffset = eventPiece[1];
    startXPos = eventPiece[2];
    startYPos = eventPiece[3];
    Xextent = eventPiece[4];
    Yextent = eventPiece[5];
    attributes = eventPiece[6];

    drawRegion.topLeft.x = (startXPos - xScrollOffset);
    drawRegion.topLeft.y = (startYPos - yScrollOffset);
    drawRegion.extent.x = Xextent;
    drawRegion.extent.y = Yextent;
  }

  if (!(attributes & IMAGE)) {
    if (event == -1 && (attributes & LINK)) {
      /* pen down event on a link area */
      /* penDownOrigImage = WinSaveBits(&drawRegion, &error); */
      penDownOrigImageFlag = 1;
      penDownOrigPieceOff = eventPieceOff;
      WinDrawRectangle(&drawRegion, 0);
      if (attributes & 0x20) {
	/* don't do this with an image */
	/* maybe fix later */
      }
      else {
	INTWinProxyStringPiece(eventPiece, xScrollOffset, yScrollOffset,
	    &drawRegion, NULL, 1);
	/* This is where we draw the URL in the status bar */
	StatusBarText((unsigned char *)&eventPiece[8], 1);
      }
      /* HiLiPiecesLink(event, eventPiece, (char *)&eventPiece[8], scrollOffset); */
      
    } else if (event == -1) {
	retval = 1;
    } else if ((event == 0 && penDownOrigPieceOff != eventPieceOff) ||
	       (event == 1)) {
      /* movement after penDown out of region OR
       * penUp condition */
      if (penDownOrigImageFlag) {
        SWord *penDownOrigPiece = hPtr + penDownOrigPieceOff;
	drawRegion.topLeft.x = (penDownOrigPiece[2] - xScrollOffset);
	drawRegion.topLeft.y = (penDownOrigPiece[3] - yScrollOffset);
	drawRegion.extent.x = penDownOrigPiece[4];
	drawRegion.extent.y = penDownOrigPiece[5];
	WinEraseRectangle(&drawRegion, 0);
	INTWinProxyStringPiece(penDownOrigPiece, xScrollOffset,
				yScrollOffset, &drawRegion, NULL, 0);
	/* This is where we erase the URL from the status bar */
	StatusBarText(NULL, 0);
	/* WinRestoreBits(penDownOrigImage, penDownOrigPiece[2],
	   penDownOrigPiece[3] - scrollOffset); */
	penDownOrigImageFlag = 0;
      }
      /* HiLiPiecesLink(event, eventPiece, (char *)&eventPiece[8], scrollOffset); */
    }
    if (penDownOrigPieceOff == eventPieceOff) {
      retval = 1;
    } else {
      penDownOrigPieceOff = -1;
      retval = 0;
    }
    if (event == 1) {
      penDownOrigPieceOff = -1;
    }
  }

  MemHandleUnlock(hPage);
  return retval;  
}

void GetCurPageDims(SWord *widthp, SWord *heightp) {
  SWord pageXextent = 0;
  SWord pageYextent = 0;
  SWord *proxyText;
  if (hPage) {
      proxyText = MemHandleLock(hPage);
      if (proxyText) {
	int readPoint = 0; /* should always be offset to
	                      a next forward offset */
	SWord forwardOffset = proxyText[readPoint+1];
	while (proxyText[readPoint] != 0xFFFF) {
	  int newpageXextent = proxyText[readPoint+2] + proxyText[readPoint+4];
	  int newpageYextent = proxyText[readPoint+3] + proxyText[readPoint+5];
	  if (pageXextent < newpageXextent) pageXextent = newpageXextent;
	  if (pageYextent < newpageYextent) pageYextent = newpageYextent;
	  readPoint += (forwardOffset / 2);
	  forwardOffset = proxyText[readPoint+1];
	}
      }
      MemHandleUnlock(hPage);
  }
  if (widthp) *widthp = pageXextent;
  if (heightp) *heightp = pageYextent;
}
 
/* this part of the code is currently unused; may be put in some other day */
 
/* relies on the hope that same links are next to each other
 * in the data structures */

#ifdef 0

void HiLiPiecesLink(int event, SWord *piece, char *linkURL, int scrollOffset) {
  INTHiLiPiecesLink(event, piece, 0, linkURL, scrollOffset);
  INTHiLiPiecesLink(event, piece, 1, linkURL, scrollOffset);
}

void INTHiLiPiecesLink(int event, SWord *eventPiece, int direction,
		       char *linkURL, int scrollOffset) {
  Word attributes;
  SWord forwardOffset = 0, backOffset = 0;

  backOffset = eventPiece[0];
  forwardOffset = eventPiece[1];

  if (direction == 0) {
    eventPiece = eventPiece - (backOffset / 2);
  } else if (direction == 1) {
    eventPiece = eventPiece + (forwardOffset / 2) - 1;
  }

  backOffset = eventPiece[0];
  forwardOffset = eventPiece[1];
  attributes = eventPiece[6];

  if ((attributes & LINK) && (StrCompare (linkURL,
    (char*)&eventPiece[((attributes & IMAGE) &&
			(attributes & IMAGE_HAS_SERV_SIZE)) ? 10 : 8]))) {
    UpdateHiLiPiece(event, eventPiece, scrollOffset);
    if (direction == 0) {
      INTHiLiPiecesLink(event, eventPiece, 0, linkURL, scrollOffset);
    } else if (direction == 1) {
      INTHiLiPiecesLink(event, eventPiece, 1, linkURL, scrollOffset);
    }
  }
}

#endif

